#  Copyright (c) 2025.
#  Penetrum LLC (all rights reserved)
#  Copyright last updated: 4/22/25, 10:49 AM
#

def output(out, filename):
    raise NotImplementedError("PDF output is not implemented yet")



#  Copyright (c) 2025.
#  Penetrum LLC (all rights reserved)
#  Copyright last updated: 5/1/25, 4:16 PM
#

import malcore_playbook.main.entry as entry


def run():
    """ CLI tool execution entrypoint """
    entry.main()



#  Copyright (c) 2025.
#  Penetrum LLC (all rights reserved)
#  Copyright last updated: 5/2/25, 10:29 AM
#
#

import re

import malcore_playbook.execution.recipe_exec as recipe_exec
import malcore_playbook.lib.settings as settings


class MalScriptParserError(Exception):

    def __init__(self, msg, code, offending_line):
        self.msg = msg
        self.code = code
        self.offending_line = offending_line
        super().__init__(msg)

    def __str__(self):
        return f"[Error][{self.code}]: {self.msg}\n\t-> Code causing crash: {self.offending_line}"


class ScriptSyntaxError(MalScriptParserError): pass


class ScriptParserError(MalScriptParserError): pass


class ScriptExecutionError(MalScriptParserError): pass


class InvalidScriptPassed(FileNotFoundError): pass


class MalScriptInterpreter(object):

    """ basic interpreter for the MalScript scripting language """

    def __init__(self, filename, **kwargs):
        self.variables = {}
        self.filename = filename
        self.kwargs = kwargs
        self.matcher = re.compile(r"if (.+?) in (.+?) then (.+)")
        self.version = "1.0"

    def exec_command(self, command, line_no, line):
        """ executes the exec() built in """
        settings.logger.info(f"Executing command: {command} on filename: {self.filename}")
        recipe = recipe_exec.load_recipe([command], load_one=True)
        if recipe is None:
            raise ScriptExecutionError(
                f"Failed to execute requested recipe: {command}, lino_no: {line_no}", -3, line
            )
        exec_results = recipe_exec.execute_recipe(recipe, self.filename, **self.kwargs)
        return exec_results

    def parse_value(self, value, line_no, line):
        """ parses and adds the variables to the storage dict """
        value = value.strip()
        type_, value = value.split("(")
        value = value.split(")")[0]
        if type_.lower() == "str":
            return value
        elif type_.lower() == "int":
            value = value.replace('"', "").replace("'", "")
            if not isinstance(value, int):
                return int(value)
            else:
                return value
        elif type_.lower() == "exec":
            command = value
            return self.exec_command(command, line_no, line)
        else:
            raise ScriptSyntaxError(
                f"Unsupported value type: {value}, line_no: {line_no}", -2, line
            )

    def get_nested_variables(self, var_name, condition_value, then_part, line_no, line, **kwargs):
        """ find nested variables data from the script """
        is_from_ret = kwargs.get("is_from_ret", False)

        if not is_from_ret:
            parts = var_name.split(".")
            var_name = parts[0]
            if var_name not in self.variables:
                raise ScriptParserError(
                    f"Requested variable: {var_name} not found, line_no: {line_no}", -1, line
                )
            data = self.variables[var_name]
            keys = parts[1:]
            for key in keys:
                if key.startswith("["):
                    try:
                        index_number = int(key.split("[")[1].split("]")[0])
                        data = data[index_number]
                    except:
                        raise ScriptSyntaxError(
                            f"Invalid index for variable: {var_name}, line_no: {line_no}", -2, line
                        )
                else:
                    if key in data.keys():
                        data = data.get(key)
                    else:
                        raise ScriptSyntaxError(
                            f"Invalid variable: {var_name}, line_no: {line_no}", -2, line
                        )
            var_value = data
            if isinstance(var_value, list):
                if any(condition_value == item for item in var_value):
                    self.parse_value(then_part, line_no, line)
            else:
                if isinstance(condition_value, int):
                    if condition_value == var_value:
                        self.parse_line(then_part, line_no)
                else:
                    if condition_value in str(var_value):
                        self.parse_line(then_part, line_no)
        else:
            parts = var_name.split(".")
            var_name = parts[0]
            keys = parts[1:]
            data = self.variables[var_name]
            for key in keys:
                if key.startswith("["):
                    try:
                        index_number = int(key.split("[")[1].split("]")[0])
                        data = data[index_number]
                    except:
                        raise ScriptSyntaxError(
                            f"Invalid index for variable: {var_name}, line_no: {line_no}", -2, line
                        )
                else:
                    if key in data.keys():
                        data = data.get(key)
                    else:
                        raise ScriptSyntaxError(
                            f"Invalid variable: {var_name}, line_no: {line_no}", -2, line
                        )
            return data

    def parse_line(self, line, line_no):
        """ parses the lines of the script """
        line_no = str(line_no)
        line = line.strip()
        if line.startswith('$') and '=' in line:
            settings.logger.debug(f"Found variable in script, parsing variable, line_no: {line_no}")
            var_name, value = line.split('=', 1)
            var_name = var_name.strip()
            value = value.strip().rstrip(';')
            if var_name in self.variables:
                settings.logger.debug(f"Variable already exists from line_no: {line_no}, overwriting")
                del self.variables[var_name]
            self.variables[var_name] = self.parse_value(value, line_no, line)

        elif line.startswith("#"):
            settings.logger.debug(f"Found a commented line on line_no: {line_no}, skipping")

        elif line.startswith('if '):
            settings.logger.debug(f"Found conditional if statement on line_no: {line_no}, parsing condition")
            match = self.matcher.match(line)
            if match:
                settings.logger.debug(f"Condition is acceptable, starting execution")
                condition_value, var_name, then_part = match.groups()
                condition_value = self.parse_value(condition_value, line_no, line)
                if isinstance(condition_value, str):
                    condition_value = condition_value.replace("'", "").replace('"', "")
                var_name = var_name.strip()
                if var_name.startswith('$'):
                    var_name = var_name
                else:
                    var_name = '$' + var_name
                then_part = then_part.strip()
                if "." in var_name:
                    self.get_nested_variables(var_name, condition_value, then_part, line_no, line)
                else:
                    if var_name not in self.variables:
                        raise ScriptParserError(
                            f"Requested variable: {var_name} not found, line_no: {line_no}", -1, line
                        )
                    var_value = self.variables[var_name]
                    if isinstance(var_value, list):
                        if any(condition_value == item for item in var_value):
                            self.parse_line(then_part, line_no)
                    else:
                        if condition_value in str(var_value):
                            self.parse_line(then_part, line_no)

        elif line.startswith('ret'):
            settings.logger.debug(f"Found return statement on line_no: {line_no}, parsing return")
            var_name = line.split("(")[1].split(")")[0].strip()
            if var_name.startswith('$'):
                var_name = var_name
            else:
                var_name = '$' + var_name
            if "." in var_name:
                return self.get_nested_variables(
                    var_name, None, None, line_no, line, is_from_ret=True
                )
            else:
                if var_name in self.variables:
                    return self.variables[var_name]
                else:
                    raise ScriptParserError(
                        f"Variable {var_name} not found, line_no: {line_no}", -1, line
                    )

        else:
            raise ScriptSyntaxError(
                f"Unknown line format: {line}, line_no: {line_no}", -2, line
            )

    def start_execution(self, script):
        """ starts execution of the script """
        settings.logger.debug(f"Starting execution of passed script")
        lines = script.strip().split(';')
        settings.logger.debug(f"Total of {len(lines)} line(s) to parse in script")
        result = None
        for i, line in enumerate(lines, start=1):
            if line != '':
                out = self.parse_line(line, i)
                if out is not None:
                    result = out
        return result


def run(script, filename, kwargs):
    """ pointer function to start the entire process """
    if kwargs is None:
        kwargs = {}
    engine = MalScriptInterpreter(filename, **kwargs)
    result = engine.start_execution(script)
    return result



#  Copyright (c) 2025.
#  Penetrum LLC (all rights reserved)
#  Copyright last updated: 5/1/25, 4:16 PM
#

import os
import sys

import malcore_playbook.lib.settings as settings


def load_recipe(recipes, load_one=False, speak=True):
    """ import the loaded recipes """
    loaded = []
    for recipe in recipes:
        if speak:
            try:
                settings.logger.info(f"Attempting to load recipe: {recipe.__name__}")
            except:
                settings.logger.info("Attempting to load recipe")
        recipe_home = settings.RECIPE_HOME
        full_path = os.path.join(recipe_home, recipe)
        path, fname = os.path.split(full_path)
        modulename, _ = os.path.splitext(fname)
        if path not in sys.path:
            if speak:
                settings.logger.debug("Path not found in sys.path adding to it")
            sys.path.insert(0, path)
        try:
            imported_mod = __import__(modulename)
            if settings.user_can_use_recipe(imported_mod.__excluded_plans__):
                loaded.append(__import__(modulename))
            else:
                if speak:
                    settings.logger.warning(
                        f"Your plan does not allow usage of recipe: {recipe}, "
                        f"to upgrade your plan see here: https://malcore.io/pricing"
                    )
        except Exception as e:
            if speak:
                settings.logger.error(f"Cannot import recipe: {recipe}, hit error: {str(e)}")
    if len(loaded) != 0:
        if load_one:
            return loaded[0]
        else:
            return loaded
    else:
        settings.logger.warning("No recipes loaded successfully")
        return None


def execute_recipe(recipe, *args, **kwargs):
    """ execute the loaded recipes and start processing them """
    settings.logger.info(f"Attempting to execute recipe: {recipe}")
    try:
        results = recipe.plugin(*args, **kwargs)
        if results is not None:
            settings.logger.debug(f"Recipe executed successfully returning results")
        return results
    except Exception as e:
        settings.logger.error(f"Unable to execute recipe, got error: {str(e)}")
        return None



from setuptools import setup, find_packages
from malcore_playbook import __version__


if __name__ == "__main__":
    setup(
        name='malcore-playbook',
        packages=find_packages(),
        version=__version__.VERSION,
        description='Malcore Playbook automates malware analysis, malware triaging, '
                    'and analyst workflows using modular recipes and DSL scripting',
        author="Thomas Perkins",
        author_email="penetrumcorp@gmail.com",
        install_requires=[l.strip() for l in open("requirements.txt").read().splitlines()],
        long_description=open("README.md").read(),
        long_description_content_type="text/markdown",
        url="https://github.com/PenetrumLLC/malcore-playbook",
        entry_points={
            'console_scripts': [
                'malcore-playbook=malcore_playbook.cli_tool:run',
                'mcpb=malcore_playbook.cli_tool:run',
            ]
        }
    )



#  Copyright (c) 2025.
#  Penetrum LLC (all rights reserved)
#  Copyright last updated: 5/2/25, 10:12 AM
#

import sys
import argparse

import malcore_playbook.lib.settings as settings


logger = settings.logger


class Parser(argparse.ArgumentParser):

    """ class placeholder for the argparse item """

    @staticmethod
    def optparse():
        """ makes it easier to call from Parser().optparse() and looks nicer """
        parser = argparse.ArgumentParser()

        parser.usage = (f"malcore-playbook --recipe RECIPE[,RECIPE,..] --filename FILE "
                        "[--chain --script [SCRIPT] "
                        "--kwargs ARG1=VAL1[,ARG2=VAL2,...]]")

        required = parser.add_argument_group("required arguments")
        required.add_argument(
            "-r", "--recipe", nargs="+", metavar="RECIPE-NAME",
            help="Recipes to execute one at a time, pass multiple using a comma seperated list ("
                 "eg, recipe1,recipe2,...)",
            default=None, dest="useRecipe"
        )
        required.add_argument(
            "-c", "--chain", action="store_true",
            dest="useChain", default=False,
            help="Pass this to chain recipes together with a script, must pass the --script flag with this"
        )
        required.add_argument(
            "--filename", "-f", "--file-to-analyze", nargs=1, default=None,
            help="The filename that you want to process with the recipes. This is required for the recipes to work",
            dest="filename"
        )

        chain_flags = parser.add_argument_group("chain related arguments")
        chain_flags.add_argument(
            "--chain-script", "-s", "--script", "-C", metavar="CHAIN-SCRIPT",
            dest="chainScript", default=None,
            help="Pass either a filename or a raw chain script in order to execute the MalScript chain"
        )

        recipe_args = parser.add_argument_group("recipe related arguments")
        recipe_args.add_argument(
            "--list-remote", "--list-remote-recipes", "-lR", action="store_true", default=False,
            help="List all remote recipes that are available for download", dest="viewRemote"
        )
        recipe_args.add_argument(
            "--list-local", "--list-local-recipes", "-lL", action="store_true", default=False,
            help="List all local recipes that are available to execute", dest="viewLocal"
        )
        recipe_args.add_argument(
            "--download-remote", "--download-recipe", "--download", "-D",
            nargs="+", metavar="RECIPE-NAME", default=None,
            help="Pass a remote recipe name to download it to your recipe folder ("
                 "pass 'all' to download all available recipes"
                 ")",
            dest="downloadRecipe"
        )
        recipe_args.add_argument(
            "--search", "-S", "--search-string",
            metavar="KEYWORD", default=None,
            help="Pass a search string to filter the local or remote recipe list",
            dest="searchString"
        )
        recipe_args.add_argument(
            "--recipe-updates", "--update-recipes", "--updates", "-U",
            metavar="ACTION", help="Check for recipe updates",
            dest="checkRecipeUpdates", default=None, choices=["check", "download"]
        )
        recipe_args.add_argument(
            "--kwargs", dest="kwargs", nargs="*", default={},
            help="Key and value pairs to pass to the recipe IE: arg1=var1,arg2=var2"
        )

        misc_args = parser.add_argument_group("misc arguments")
        misc_args.add_argument(
            "--force", action="store_true", default=False,
            help="Force actions that would otherwise fail", dest="forceAction"
        )
        misc_args.add_argument(
            "--output", "-O", "--output-type",
            default="json", choices=["json", "pdf", "txt", "console"],
            metavar="OUTPUT-TYPE", dest="outputType",
            help=f"Pass to control the type of output you want, default is JSON files stored in: {settings.HOME}"
        )
        misc_args.add_argument(
            "--hide", action="store_true", help="Hide the banner", dest="hideBanner"
        )
        misc_args.add_argument(
            "--version", action="store_true", help="Show version numbers and exit",
            dest="showVersions"
        )

        # Hidden args
        parser.add_argument(
            "--no-start-end", action="store_true", default=False, dest="noStartEnd",
            help=argparse.SUPPRESS
        )
        parsed = parser.parse_args()

        # if any of these are passed we will go ahead and hide the banner and prevent the
        # startup and shutdown logging
        if parsed.viewLocal or parsed.viewRemote or parsed.showVersions:
            parsed.hideBanner = True
            parser.noStartEnd = True

        # process KEY=VAL pairs from the --kwargs argument and put them into a dict
        # for future use
        kwargs_dict = {}
        for item in parsed.kwargs:
            if "=" in item:
                key, value = item.split("=")
                kwargs_dict[key] = value
            else:
                logger.warning(f"Key value pair: {item} will be skipped")
        parsed.kwargs = kwargs_dict

        return parsed

